from __future__ import annotations

import json
import logging

from django.apps import apps
from django.core.management.base import BaseCommand
from django.db import models

from django_qstash.callbacks import get_callback_url
from django_qstash.client import qstash_client

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    """Management command to list and sync QStash schedules."""

    help = "List and sync schedules from QStash"

    def add_arguments(self, parser) -> None:
        parser.add_argument(
            "--list",
            action="store_true",
            help="List schedules from QStash",
        )
        parser.add_argument(
            "--sync",
            action="store_true",
            help="Sync schedules from QStash to local database",
        )
        parser.add_argument(
            "--no-input",
            action="store_true",
            help="Do not ask for confirmation",
        )

    def get_task_schedule_model(self) -> models.Model | None:
        """Get the TaskSchedule model if available."""
        try:
            return apps.get_model("django_qstash_schedules", "TaskSchedule")
        except LookupError:
            self.stdout.write(
                self.style.ERROR(
                    "Django QStash Schedules not installed.\n"
                    "Add `django_qstash.schedules` to INSTALLED_APPS and run migrations."
                )
            )

    def sync_schedules(self, schedules: list) -> None:
        """Sync remote schedules to local database."""
        TaskSchedule = self.get_task_schedule_model()

        for schedule in schedules:
            try:
                body = json.loads(schedule.body)
                task_name = body.get("task_name", "Unnamed Task")
                function = f"{body['module']}.{body['function']}"

                obj, created = TaskSchedule.objects.update_or_create(
                    schedule_id=schedule.schedule_id,
                    defaults={
                        "name": task_name,
                        "task": function,
                        "cron": schedule.cron,
                        "args": body.get("args", []),
                        "kwargs": body.get("kwargs", {}),
                    },
                )
                status = "Created" if created else "Updated"
                logger.info(
                    "%s schedule: %s (%s)", status, task_name, schedule.schedule_id
                )
            except Exception:
                logger.exception("Failed to sync schedule %s", schedule.schedule_id)

    def handle(self, *args, **options) -> None:
        auto_confirm = options.get("no_input")
        if not (options.get("sync") or options.get("list")):
            self.stdout.write(
                self.style.ERROR("Please specify either --list or --sync option")
            )
            return

        try:
            destination = get_callback_url()
            schedules = qstash_client.schedule.list()

            self.stdout.write(
                self.style.SUCCESS(
                    f"Found {len(schedules)} remote schedules based on destination: {destination}"
                )
            )

            for schedule in schedules:
                body = json.loads(schedule.body)
                task_name = body.get("task_name", "Unnamed Task")
                function = f"{body['module']}.{body['function']}"

                self.stdout.write(
                    f"\nSchedule ID: {schedule.schedule_id}"
                    f"\n  Task: {task_name} ({function})"
                    f"\n  Cron: {schedule.cron}"
                    f"\n  Destination: {schedule.destination}"
                    f"\n  Retries: {schedule.retries}"
                    f"\n  Status: {'Paused' if schedule.paused else 'Active'}"
                )

            if options.get("sync"):
                user_input = input("Do you want to sync remote schedules? (y/n): ")
                if user_input.lower() == "y" or auto_confirm:
                    self.sync_schedules(schedules)
                else:
                    self.stdout.write(self.style.ERROR("Sync cancelled"))
        except Exception as e:
            self.stdout.write(self.style.ERROR(f"An error occurred: {str(e)}"))



from __future__ import annotations

import json
from unittest.mock import patch

import pytest
from django.test import Client


@pytest.mark.django_db
class TestQStashWebhook:
    def setup_method(self):
        self.client = Client()
        self.url = "/qstash/webhook/"

    @patch("django_qstash.views.QStashWebhook")
    def test_valid_webhook_request(self, mock_webhook_class):
        """Test webhook with valid signature and payload"""
        # Setup mock webhook instance
        mock_webhook = mock_webhook_class.return_value
        mock_webhook.handle_request.return_value = ({"status": "success"}, 200)

        payload = {
            "function": "sample_task",
            "module": "tests.test_tasks",
            "args": [2, 3],
            "kwargs": {},
            "task_name": "test_task",
        }

        response = self.client.post(
            self.url,
            data=json.dumps(payload),
            content_type="application/json",
            headers={"upstash-signature": "mock-signature"},
        )

        # Verify response
        assert response.status_code == 200
        response_data = json.loads(response.content)
        assert response_data["status"] == "success"

        # Verify webhook was called correctly
        mock_webhook.handle_request.assert_called_once()

    @patch("django_qstash.views.QStashWebhook")
    def test_invalid_request(self, mock_webhook_class):
        """Test webhook with invalid request"""
        # Setup mock webhook instance
        mock_webhook = mock_webhook_class.return_value
        mock_webhook.handle_request.return_value = ({"error": "Invalid request"}, 400)

        response = self.client.post(
            self.url,
            data="invalid json",
            content_type="application/json",
            headers={"upstash-signature": "mock-signature"},
        )

        assert response.status_code == 400
        response_data = json.loads(response.content)
        assert "error" in response_data



from __future__ import annotations

from django.contrib import admin

from django_qstash.schedules import services
from django_qstash.schedules.forms import TaskScheduleForm
from django_qstash.schedules.models import TaskSchedule


@admin.register(TaskSchedule)
class TaskScheduleAdmin(admin.ModelAdmin):
    list_display = ["schedule_id", "task_name"]
    readonly_fields = [
        "schedule_id",
        "task_name",
        "get_qstash_schedule_details",
        "paused_at",
        "resumed_at",
        "active_at",
    ]
    form = TaskScheduleForm

    fieldsets = [
        (
            "Name",
            {
                "fields": ["name", "schedule_id", "is_active"],
            },
        ),
        (
            "Task Selection",
            {
                "fields": ["task", "task_name"],
            },
        ),
        (
            "Arguments",
            {
                "fields": ["args", "kwargs"],
            },
        ),
        (
            "Schedule",
            {
                "fields": ["cron", "retries", "timeout"],
            },
        ),
        (
            "QStash Metadata",
            {
                "fields": [
                    "paused_at",
                    "resumed_at",
                    "active_at",
                    "get_qstash_schedule_details",
                ],
                "classes": ["collapse"],
            },
        ),
    ]

    @admin.display(description="Raw")
    def get_qstash_schedule_details(self, obj: TaskSchedule) -> dict:
        if not obj.schedule_id:
            return "No schedule ID yet"
        return services.get_task_schedule_from_qstash(obj, as_dict=True)



from __future__ import annotations

from django.apps import AppConfig


class SchedulesConfig(AppConfig):
    name = "django_qstash.schedules"
    label = "django_qstash_schedules"
    verbose_name = "django_qstash_schedules"
    default_auto_field = "django.db.models.BigAutoField"

    def ready(self):
        import django_qstash.schedules.signals  # noqa



from __future__ import annotations

import uuid

from django.db import models
from django.utils import timezone

from django_qstash.db.models import TaskStatus


class TaskResult(models.Model):
    id = models.UUIDField(
        primary_key=True, default=uuid.uuid1, editable=False, unique=True
    )
    task_id = models.CharField(max_length=255, db_index=True)
    task_name = models.CharField(max_length=255)
    status = models.CharField(
        max_length=50,
        choices=TaskStatus.choices,
        default=TaskStatus.PENDING,
    )
    date_created = models.DateTimeField(default=timezone.now)
    date_done = models.DateTimeField(null=True)
    result = models.JSONField(null=True)
    traceback = models.TextField(blank=True, null=True)
    function_path = models.TextField(blank=True, null=True)
    args = models.JSONField(null=True)
    kwargs = models.JSONField(null=True)

    class Meta:
        app_label = "django_qstash_results"
        ordering = ["-date_done"]

    def __str__(self):
        return f"{self.task_name} ({self.task_id})"



from __future__ import annotations

from django.core.exceptions import ValidationError

from django_qstash.discovery.utils import discover_tasks


def task_exists_validator(task_name):
    """
    Validates that a task name exists in the discovered tasks

    Args:
        task_name: The name of the task to validate

    Raises:
        ValidationError: If the task cannot be found
    """
    discover_tasks.cache_clear()
    available_tasks = discover_tasks(locations_only=True)

    if task_name not in available_tasks:
        raise ValidationError(
            f"Task '{task_name}' not found. Available tasks: {', '.join(available_tasks)}"
        )



from __future__ import annotations

import re

from django.utils.safestring import mark_safe

from django_qstash import cron
from django_qstash.schedules.exceptions import InvalidCronStringValidationError
from django_qstash.schedules.exceptions import InvalidDurationStringValidationError


def validate_duration_string(value):
    if not re.match(r"^\d+[smhd]$", value):
        raise InvalidDurationStringValidationError(
            'Invalid duration format. Must be a number followed by s (seconds), m (minutes), h (hours), or d (days). E.g., "60s", "5m", "2h", "7d"'
        )

    # Extract number and unit
    number = int(value[:-1])
    unit = value[-1]

    # Convert to days
    days = {
        "s": number / (24 * 60 * 60),  # seconds to days
        "m": number / (24 * 60),  # minutes to days
        "h": number / 24,  # hours to days
        "d": number,  # already in days
    }[unit]

    if days > 7:
        raise InvalidDurationStringValidationError(
            "Duration too long. Maximum allowed: 7 days (equivalent to: 604800s, 10080m, 168h, 7d)"
        )


def validate_cron_expression(value: str) -> None:
    """Validates a standard cron expression with 5 fields (minute, hour, day of month, month, day of week)."""
    parts = value.split()
    if len(parts) != 5:
        raise InvalidCronStringValidationError(
            'Invalid cron format. Must contain 5 fields: "minute hour day_of_month month day_of_week"'
        )

    field_descriptions = {
        "minute": "0-59",
        "hour": "0-23",
        "day_of_month": "1-31",
        "month": "1-12",
        "day_of_week": "0-6 (0=Sunday, 6=Saturday)",
    }
    field_label = {
        "minute": "minute",
        "hour": "hour",
        "day_of_month": "day of the month",
        "month": "month",
        "day_of_week": "day of the week",
    }
    patterns = {
        "minute": cron.minute_re,
        "hour": cron.hour_re,
        "day_of_month": cron.day_of_month_re,
        "month": cron.month_re,
        "day_of_week": cron.day_of_week_re,
    }

    for sub_value, (field, pattern) in zip(parts, patterns.items()):
        if not re.match(pattern, sub_value):
            invalid_msg = f"""<b>{sub_value}</b> is not a valid {field_label[field]}. <br/>Must be in range {field_descriptions[field]}. <br/><a target="_blank" href="https://crontab.guru/">crontab.guru</a> is also helpful."""
            raise InvalidCronStringValidationError(mark_safe(invalid_msg))



from __future__ import annotations

import warnings

from django.test import TestCase
from django.test import override_settings

from django_qstash.settings import DJANGO_QSTASH_WEBHOOK_PATH


class SettingsTestCase(TestCase):
    def test_default_webhook_path(self):
        """Test that default webhook path is set correctly"""
        self.assertEqual(DJANGO_QSTASH_WEBHOOK_PATH, "/qstash/webhook/")

    def test_warning_when_required_settings_missing(self):
        """Test that warning is raised when required settings are missing"""
        with override_settings(QSTASH_TOKEN=None, DJANGO_QSTASH_DOMAIN=None):
            with warnings.catch_warnings(record=True) as w:
                warnings.simplefilter("always")

                # Force reload of settings to trigger warning
                from importlib import reload

                import django_qstash.settings

                reload(django_qstash.settings)

                self.assertEqual(len(w), 1)
                self.assertTrue(issubclass(w[0].category, RuntimeWarning))
                self.assertIn(
                    "QSTASH_TOKEN and DJANGO_QSTASH_DOMAIN should be set",
                    str(w[0].message),
                )

    @override_settings(QSTASH_TOKEN="test-token", DJANGO_QSTASH_DOMAIN="example.com")
    def test_no_warning_when_settings_present(self):
        """Test that no warning is raised when required settings are present"""
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")

            # Force reload of settings
            from importlib import reload

            import django_qstash.settings

            reload(django_qstash.settings)

            self.assertEqual(len(w), 0)

    @override_settings(DJANGO_QSTASH_WEBHOOK_PATH="/custom/webhook/path/")
    def test_custom_webhook_path(self):
        """Test that custom webhook path can be set"""
        # Force reload of settings to get new webhook path
        from importlib import reload

        import django_qstash.settings

        reload(django_qstash.settings)

        self.assertEqual(
            django_qstash.settings.DJANGO_QSTASH_WEBHOOK_PATH, "/custom/webhook/path/"
        )



# Generated by Django 5.1.4 on 2025-01-02 07:44

from __future__ import annotations

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("django_qstash_schedules", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="taskschedule",
            name="updated_at",
            field=models.DateTimeField(auto_now=True),
        ),
    ]



from __future__ import annotations

import functools
from typing import Any
from typing import Callable

from django.core.exceptions import ImproperlyConfigured

from django_qstash.callbacks import get_callback_url
from django_qstash.client import qstash_client
from django_qstash.settings import DJANGO_QSTASH_DOMAIN
from django_qstash.settings import QSTASH_TOKEN


class QStashTask:
    def __init__(
        self,
        func: Callable | None = None,
        name: str | None = None,
        delay_seconds: int | None = None,
        deduplicated: bool = False,
        **options: dict[str, Any],
    ):
        self.func = func
        self.name = name or (func.__name__ if func else None)
        self.delay_seconds = delay_seconds
        self.deduplicated = deduplicated
        self.options = options

        if func is not None:
            functools.update_wrapper(self, func)

    def __get__(self, obj, objtype):
        """Support for instance methods"""
        return functools.partial(self.__call__, obj)

    def __call__(self, *args, **kwargs):
        """
        Execute the task, either directly or via QStash based on context
        """
        if not QSTASH_TOKEN or not DJANGO_QSTASH_DOMAIN:
            raise ImproperlyConfigured(
                "QSTASH_TOKEN and DJANGO_QSTASH_DOMAIN must be set to use django-qstash"
            )
        # Handle the case when the decorator is used without parameters
        if self.func is None:
            return self.__class__(
                args[0],
                name=self.name,
                delay_seconds=self.delay_seconds,
                deduplicated=self.deduplicated,
                **self.options,
            )

        # If called directly (not through delay/apply_async), execute the function
        if not getattr(self, "_is_delayed", False):
            return self.func(*args, **kwargs)

        # Reset the delayed flag
        self._is_delayed = False

        # Prepare the payload
        payload = {
            "function": self.func.__name__,
            "module": self.func.__module__,
            "args": args,  # Send args as-is
            "kwargs": kwargs,
            "task_name": self.name,
            "options": self.options,
        }

        url = get_callback_url()
        # Send to QStash using the official SDK
        response = qstash_client.message.publish_json(
            url=url,
            body=payload,
            delay=f"{self.delay_seconds}s" if self.delay_seconds else None,
            retries=self.options.get("max_retries", 3),
            content_based_deduplication=self.deduplicated,
        )
        # Return an AsyncResult-like object for Celery compatibility
        return AsyncResult(response.message_id)

    def delay(self, *args, **kwargs) -> AsyncResult:
        """Celery-compatible delay() method"""
        self._is_delayed = True
        return self(*args, **kwargs)

    def apply_async(
        self,
        args: tuple | None = None,
        kwargs: dict | None = None,
        countdown: int | None = None,
        **options: dict[str, Any],
    ) -> AsyncResult:
        """Celery-compatible apply_async() method"""
        self._is_delayed = True
        if countdown is not None:
            self.delay_seconds = countdown
        self.options.update(options)

        # Fix: Ensure we're passing the arguments correctly
        args = args or ()
        kwargs = kwargs or {}
        return self(*args, **kwargs)


class AsyncResult:
    """Minimal Celery AsyncResult-compatible class"""

    def __init__(self, task_id: str):
        self.task_id = task_id

    def get(self, timeout: int | None = None) -> Any:
        """Simulate Celery's get() method"""
        raise NotImplementedError("QStash doesn't support result retrieval")

    @property
    def id(self) -> str:
        return self.task_id



from __future__ import annotations

from django.core.management.base import BaseCommand

from django_qstash.discovery.utils import discover_tasks


class Command(BaseCommand):
    help = "View all available tasks"

    def add_arguments(self, parser):
        parser.add_argument(
            "--locations",
            action="store_true",
            help="Only show task paths",
        )

    def handle(self, *args, **options):
        locations_only = options["locations"] or False
        self.stdout.write("Available tasks:")
        discover_tasks.cache_clear()
        if locations_only:
            tasks = discover_tasks(locations_only=locations_only)
            for task in tasks:
                self.stdout.write(f"\t- {self.style.SQL_FIELD(task)}")
        else:
            tasks = discover_tasks(locations_only=False)
            for task in tasks:
                name = task["name"]
                field_label = task["field_label"]
                location = task["location"]
                self.stdout.write(
                    f"  Name: {self.style.SQL_FIELD(name)}\n"
                    f"  Location: {self.style.SQL_FIELD(location)}\n"
                    f"  Field Label: {self.style.SQL_FIELD(field_label)}"
                )
                self.stdout.write("")



# Generated by Django 5.1.4 on 2025-01-13 19:27

from __future__ import annotations

from django.db import migrations
from django.db import models

import django_qstash.schedules.validators


class Migration(migrations.Migration):
    dependencies = [
        ("django_qstash_schedules", "0002_taskschedule_updated_at"),
    ]

    operations = [
        migrations.AlterField(
            model_name="taskschedule",
            name="cron",
            field=models.CharField(
                default="*/5 * * * *",
                help_text="Cron expression for scheduling the task",
                max_length=255,
                validators=[
                    django_qstash.schedules.validators.validate_cron_expression
                ],
                verbose_name="Cron Expression",
            ),
        ),
    ]



# Generated by Django 5.1.4 on 2025-01-06 05:39

from __future__ import annotations

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("django_qstash_results", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="taskresult",
            name="function_path",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="taskresult",
            name="status",
            field=models.CharField(
                choices=[
                    ("PENDING", "Pending"),
                    ("SUCCESS", "Success"),
                    ("EXECUTION_ERROR", "Execution Error"),
                    ("INTERNAL_ERROR", "Internal Error"),
                    ("OTHER_ERROR", "Other Error"),
                    ("UNKNOWN", "Unknown"),
                ],
                default="PENDING",
                max_length=50,
            ),
        ),
        migrations.AlterField(
            model_name="taskresult",
            name="task_id",
            field=models.CharField(db_index=True, max_length=255),
        ),
        migrations.AlterField(
            model_name="taskresult",
            name="traceback",
            field=models.TextField(blank=True, null=True),
        ),
    ]



from __future__ import annotations

from django.contrib import admin

from .models import TaskResult


@admin.register(TaskResult)
class TaskResultAdmin(admin.ModelAdmin):
    readonly_fields = [
        "task_name",
        "status",
        "date_done",
        "result",
        "traceback",
        "args",
        "kwargs",
        "task_id",
        "date_created",
        "function_path",
    ]
    search_fields = ["task_name", "task_id", "function_path"]
    list_display = ["task_name", "function_path", "status", "date_done"]
    list_filter = ["status", "date_done"]



from __future__ import annotations

from django.db import models


class TaskStatus(models.TextChoices):
    PENDING = "PENDING", "Pending"
    SUCCESS = "SUCCESS", "Success"
    EXECUTION_ERROR = "EXECUTION_ERROR", "Execution Error"
    INTERNAL_ERROR = "INTERNAL_ERROR", "Internal Error"
    OTHER_ERROR = "OTHER_ERROR", "Other Error"
    UNKNOWN = "UNKNOWN", "Unknown"



from __future__ import annotations

from unittest.mock import patch

import pytest

from django_qstash.callbacks import get_callback_url


@pytest.mark.parametrize(
    "domain,webhook_path,expected",
    [
        # Domain without protocol
        (
            "example.com",
            "webhook",
            "https://example.com/webhook/",
        ),
        # Domain with http protocol
        (
            "http://example.com",
            "webhook",
            "http://example.com/webhook/",
        ),
        # Domain with https protocol
        (
            "https://example.com",
            "webhook",
            "https://example.com/webhook/",
        ),
        # Domain with trailing slash
        (
            "example.com/",
            "webhook",
            "https://example.com/webhook/",
        ),
        # Webhook path with leading slash
        (
            "example.com",
            "/webhook",
            "https://example.com/webhook/",
        ),
        # Webhook path with trailing slash
        (
            "example.com",
            "webhook/",
            "https://example.com/webhook/",
        ),
        # Complex path
        (
            "example.com",
            "api/v1/webhook",
            "https://example.com/api/v1/webhook/",
        ),
    ],
)
def test_get_callback_url(domain, webhook_path, expected):
    with (
        patch("django_qstash.callbacks.DJANGO_QSTASH_DOMAIN", domain),
        patch("django_qstash.callbacks.DJANGO_QSTASH_WEBHOOK_PATH", webhook_path),
    ):
        assert get_callback_url() == expected



from __future__ import annotations

from django.core.exceptions import ImproperlyConfigured

from django_qstash.settings import DJANGO_QSTASH_DOMAIN
from django_qstash.settings import DJANGO_QSTASH_WEBHOOK_PATH


def get_callback_url() -> str:
    """
    Get the callback URL based on the settings.
    """
    if DJANGO_QSTASH_DOMAIN is None:
        raise ImproperlyConfigured("DJANGO_QSTASH_DOMAIN is not set")
    if DJANGO_QSTASH_WEBHOOK_PATH is None:
        raise ImproperlyConfigured("DJANGO_QSTASH_WEBHOOK_PATH is not set")
    callback_domain = DJANGO_QSTASH_DOMAIN.rstrip("/")
    if not callback_domain.startswith(("http://", "https://")):
        callback_domain = f"https://{callback_domain}"
    webhook_path = DJANGO_QSTASH_WEBHOOK_PATH.strip("/")
    return f"{callback_domain}/{webhook_path}/"



from __future__ import annotations

from django.core.exceptions import ValidationError


class InvalidDurationStringValidationError(ValidationError):
    """Invalid duration string."""

    pass


class InvalidCronStringValidationError(ValidationError):
    """Invalid cron string."""

    pass



from __future__ import annotations

import json
import logging
from typing import Any

from django.apps import apps
from django.utils import timezone

from django_qstash.db.models import TaskStatus

logger = logging.getLogger(__name__)


def function_result_to_json(result: Any) -> str:
    """Convert a function result to a JSON string"""
    try:
        data = {"result": result}
        return json.dumps(data)
    except Exception as e:
        logger.exception("Failed to convert function result to JSON: %s", str(e))
        return None


def store_task_result(
    task_id,
    task_name,
    status,
    result=None,
    traceback=None,
    args=None,
    kwargs=None,
    error=None,
    function_path=None,
):
    """Helper function to store task results if the results app is installed"""
    if status not in TaskStatus.values:
        status = TaskStatus.UNKNOWN

    try:
        TaskResult = apps.get_model("django_qstash_results", "TaskResult")
        task_result = TaskResult.objects.create(
            task_id=task_id,
            task_name=task_name,
            status=status,
            date_done=timezone.now(),
            result=function_result_to_json(result),
            traceback=traceback,
            args=args,
            kwargs=kwargs,
            function_path=function_path,
        )
        return task_result
    except LookupError:
        # Model isn't installed, skip storage
        logger.debug(
            "Django QStash Results not installed. Add `django_qstash.results` to INSTALLED_APPS and run migrations."
        )
        return None



from __future__ import annotations

from typing import Any
from typing import Callable

from django_qstash.app.base import QStashTask


def stashed_task(
    func: Callable | None = None,
    name: str | None = None,
    deduplicated: bool = False,
    **options: dict[str, Any],
) -> QStashTask:
    """
    Decorator that mimics Celery's shared_task that maintains
    Celery compatibility.

    Can be used as:

        from django_qstash import shared_task

        @shared_task
        def my_task():
            pass

        @stashed_task(name="custom_name", deduplicated=True)
        def my_task():
            pass
    """
    if func is not None:
        return QStashTask(func, name=name, deduplicated=deduplicated, **options)
    return lambda f: QStashTask(f, name=name, deduplicated=deduplicated, **options)


def shared_task(func: Callable | None = None, **options: dict[str, Any]) -> QStashTask:
    """
    Decorator that is a drop-in replacement for Celery's shared_task.

    Can be used as:

        from django_qstash import shared_task

        @shared_task
        def my_task():
            pass
    """
    return stashed_task(func, **options)



from __future__ import annotations

from django.apps import AppConfig


class ResultsConfig(AppConfig):
    name = "django_qstash.results"
    label = "django_qstash_results"
    verbose_name = "django_qstash_results"
    default_auto_field = "django.db.models.BigAutoField"



from __future__ import annotations

from django.db import models

from django_qstash.discovery.fields import TaskChoiceField


class TaskField(models.CharField):
    """
    A model field for storing QStash task references
    """

    def __init__(self, *args, **kwargs):
        # Set a reasonable max_length for task names if not provided
        if "max_length" not in kwargs:
            kwargs["max_length"] = 255
        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        # Use our custom form field
        defaults = {
            "form_class": TaskChoiceField,
        }
        defaults.update(kwargs)
        return super().formfield(**defaults)



from __future__ import annotations


class WebhookError(Exception):
    """Base exception for webhook handling errors."""

    pass


class SignatureError(WebhookError):
    """Invalid or missing signature."""

    pass


class PayloadError(WebhookError):
    """Invalid payload structure or content."""

    pass


class TaskError(WebhookError):
    """Error in task execution."""

    pass
